"""
    Copyright 2006,2007 Riccardo Taormina (den3b@hotmail.com)
    
    This file is part of SmartChameleons.

    SmartChameleons is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    SmartChameleons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SmartChameleons; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

from numpy import oldnumeric
from numpy import *
import random

def phi(x,a=1):
    # this is the activation function for all the neurons in the net
    # x is an array, so it will be the output. The output will be the
    # result of the activation function for all the neurons in a non-input layer
    output = 1/(1 + exp(-a*x))
    return output

class Net:
    def __init__(self,m,weightCoeff=0.3):
        self.weightCoeff=weightCoeff
        self.neuronsForLayer = array(m)
        self.nLayers = len(m)
        self.learningRates=[1]*(self.nLayers-1)
        self.momentums=[0.25]*(self.nLayers-1)

        random.seed(12345)
        
        self.weightMatrixs=[]
        self.oldChanges = []
        for i in range(self.nLayers-1):
        # we create a WEIGHTS MATRIX with as many rows as the CURRENT LAYER neurons and
        # with as many columns as the PREVIOUS LAYER neurons
        # self.neuronsForLayer[i+1] because INPUT LAYER has no connections, SHIFTING NEEDED
        # self.neuronsForLayer[i]+1 because there is the BIAS in prev.Layer
            nRows = self.neuronsForLayer[i+1]
            nColumns = self.neuronsForLayer[i]+1
            matrix=zeros((nRows,nColumns))
            for j in range(nRows):
                for k in range(nColumns):
                    matrix[j,k]=self.generateWeight()
            self.weightMatrixs.append(matrix)
            self.oldChanges.append(matrix*0)
    
    def generateWeight(self):
        weight = (random.random()-0.5)*(self.weightCoeff*2)
        return weight

    def forward(self,pattern):
        # we put the pattern in the "values" array because we need it to automate forwarding and training,
        # these are the values of the INPUT neurons
        values = []
        values.append(pattern)
        # nLayers-1 below is because we now compute the values for the hidden layers and the output layer
        for i in range(self.nLayers-1):
            matrix = self.weightMatrixs[i] # we select the weight matrix for the current LAYER
            # values[i] because weightMatrixs is SHIFTED ONE POSITION BACK in respect to the VALUES ARRAY
            localField = dot(matrix,concatenate((ones(1),values[i]))) # computation of the localFields of the CURRENT LAYER NEURONS
            #print "This is the local Field of layer num. ", i+1, " ", localField
            temp = phi(localField)
            values.append(temp)
        return values.pop(), values


    def backdrop(self,o, y, d, a = 1, eta = 1, alpha = 0):
        changes = []
        for i in range(len(self.weightMatrixs)):
            changes.append(self.weightMatrixs[i]*0)
        gradients = [[]]*(self.nLayers-1)
        # starting from the output layer
        gradients[self.nLayers-2] = a*(d-o)*o*(1-o)
        for i in range(self.nLayers-2,0,-1):
            gradients[i-1] = (a*y[i]*(1-y[i]))*dot(gradients[i],self.weightMatrixs[i][:,1:])
        # let us CHANGE the WEIGHTS
        for i in range(self.nLayers-1):
            changes[i] = self.momentums[i]*self.oldChanges[i] + self.learningRates[i] * oldnumeric.outerproduct(gradients[i],concatenate((ones(1),y[i])))
            self.weightMatrixs[i] += changes[i]
        self.oldChanges = changes[:]

    def mapping(self,x):
        biggest = float(abs(x).max())
        return x/biggest,biggest

    def sorting(self,patterns,targets):
        data=range(len(patterns))
        tempPatterns = patterns[:]
        tempTargets = targets[:]
        i = 0
        random.shuffle(data)
        for elem in data:
            tempPatterns[i] = patterns[elem]
            tempTargets[i] = targets[elem]
            i += 1
        return tempPatterns, tempTargets

    def loadWeights(self,weights):
        self.weightMatrixs=weights

    def ask(self,pattern):
        [output,otherValues]=self.forward(pattern)
        return output
        



