"""
    Copyright 2006,2007 Riccardo Taormina (den3b@hotmail.com)
    
    This file is part of SmartChameleons.

    SmartChameleons is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    SmartChameleons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SmartChameleons; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import random

from Tkinter import *
from Panel import *
from tkSimpleDialog import *
from Model import *
from OutputWindow import *
from ActionGroup import ActionGroup
import tkMessageBox

class Observer(Tk):
    def __init__(self):
        Tk.__init__(self)

        self.title("Observer Environment")
        #loading the chameleons images
        self.cham_images={"blue":PhotoImage(file='blue.gif'),
                          "red":PhotoImage(file='red.gif'),
                          "green":PhotoImage(file='green.gif')}

        self.result = None #this will be filled by the "input parameters" Dialog Box
        self.stopIt = False
        self.onlyOneColor = False
        self.speed = 1
        self.refreshFrequency = 1
        self.forceExit = False
        self.smartColor = ''

        self.outputToFile = []

        #StringVar required fot the GUI Output window.
        self.blueNumber = StringVar()
        self.redNumber = StringVar()
        self.greenNumber = StringVar()
        self.cycles = StringVar()

        self.protocol("WM_DELETE_WINDOW", self.handler)
        

    def buildObjects(self):
        #build the GUI components
        self.panel = Panel(self)
        self.output = OutputWindow(self)
        self.dialog = Dialog(self)

    def create(self):
        self.maxX=self.result[0]
        self.maxY=self.result[1]
        self.n_cycles=self.result[2]
        self.n_chams=self.result[3]
        self.seed = self.result[4]

        random.seed(random.seed)

        #create the canvas showing the environment
        self.canvas = Canvas(self,background = "white",width=self.maxX*36,height=self.maxY*36)
        self.canvas.pack()

        #build the Model
        self.model = Model(self,self.n_chams,["blue","red","green"],self.n_cycles,self.maxX,self.maxY)
        self.model.buildObjects()
        self.model.buildActions()


    def buildActions(self):
    

        self.refreshActionGroup = ActionGroup ("refresh")

        def refreshOutput(self):
            chams_x_color = [0,0,0]
            for i in range(3):
                for chameleon in self.model.chList:
                    if chameleon.color == self.model.colorList[i]:
                        chams_x_color[i] += 1
            self.blueNumber.set(str(chams_x_color[0]))
            self.redNumber.set(str(chams_x_color[1]))
            self.greenNumber.set(str(chams_x_color[2]))
            self.cycles.set(str(self.model.time)+"/"+str(self.n_cycles))
            
            self.outputToFile.append(str(chams_x_color[0]) + " " + str(chams_x_color[1]) + " "\
                           + str(chams_x_color[2]) + "\n")

        def refreshEnvironment(self):
            if self.model.time % self.refreshFrequency == 0 :
                for chameleon in self.model.chList:
                    item = chameleon.graphItem
                    self.canvas.coords(item,chameleon.xPosition*36,chameleon.yPosition*36)
                    self.update()
        
        def insertDelay(self):
            if self.speed < 5:
                self.after(50/self.speed)

        self.refreshActionGroup.actions = [refreshOutput,refreshEnvironment,insertDelay]

        self.actionGroupList = ["step", "refresh"]


    def go(self):

        [modelIsRunning, message] = self.checkUp()

        while modelIsRunning:
            for action in self.actionGroupList:
                if action == "step":
                    self.model.step()
                elif action == "refresh":
                    for refreshAction in self.refreshActionGroup.actions:
                        refreshAction(self)

            [modelIsRunning, message] = self.checkUp()

        if message != 'stopped by user':
            if message == "exit forced":
                self.destroy()
            else :
                tkMessageBox.showwarning("Message", message)
                self.storeOutput()
                self.destroy()


    def handler(self):
        self.panel.focus_set()
        self.withdraw()


    def quit(self):
        if self.model.time == 0 or self.model.time==self.n_cycles or self.stopIt or self.onlyOneColor:
            self.destroy()
        else :
            self.forceExit=True


    def checkUp(self):
        flag = True
        message = ''
        if self.onlyOneColor :
            self.panel.go_b["state"]=DISABLED
            message = 'Chameleons stopped changing at cycle num. ' + str(self.model.time) + \
                      '; ' + self.model.chList[0].color + ' wins'
            flag = False

        elif self.model.time == self.n_cycles :
            self.panel.go_b["state"]=DISABLED
            message = str(self.n_cycles) + ' have passed. Neither color is the winner.'
            flag = False

        elif self.forceExit :
            message = 'exit forced'
            flag = False

        elif self.stopIt :
            message = 'stopped by user'
            flag = False

        else :
            message = "YES"

        return flag,message


    def storeOutput(self):
        i=0
        while 1:
            path='output'
            if i == 0:
                add = '.txt'
            else :
                add = str(i) + '.txt'
            path = path + add
            try :
                file_=open(path,'r')
                file_.close()
                i+=1
                continue
            except IOError:
                file_=open(path,'w')
                file_.writelines(self.outputToFile)
                file_.close
                break
            
            
            
            




        
        
